// file eclcd.c
#include "ecash.h"

/*
; LCD Display Driver for one line X 16 character LCD module in eCash system

; Assumes that LCD data bus is CPU Port 2, ENABLE strobe is P3.6,
; Data/Control Mode = P3.7, and Read/Write mode = P3.5.
*/

#define ASCII_LF 0x0A
#define ASCII_FF 0x0C
#define ASCII_CR 0x0D
#define ASCII_SPACE 0x20
#define ASCII_LIM ('9'+1)

#define MAX_CHARCOUNT 8

#define USE_LCD 1

// Port to use for Data Bus
#define DATA_BUS P2
// set bit for DATA, clear for CONTROL
DECLARE_BIT(P3, 7, DATA_CONTROL);
// strobe bit for data/control ready
DECLARE_BIT(P3, 6, ENABLE);
// set bit for READ, clear bit for WRITE
DECLARE_BIT(P3, 5, READ_WRITE);

void lcd_init(void);
void lcd_putstring(uchar* message, uchar len);

void lcd_control(uchar controlByte);
void lcd_putchar(uchar dataByte);

xbyte lcdCharCount;


/*
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Function:	LCD_Init
;
; Desc:		Prepare the LCD display for use
*/
void lcd_init(void)
{
#if USE_LCD
   ENABLE = 0;
   READ_WRITE = 0;
   usDelay(1600);

   DATA_CONTROL = 0;

   //lcd_control(0x30);
   // 2-line mode, 5x7 dots
   lcd_control(0x3C);
   usDelay(50);
   // 2-line mode, 5x7 dots
   lcd_control(0x3C);
   usDelay(50);
   //wait43us();
   // 2-line mode, 5x7 dots
   lcd_control(0x3C);
   usDelay(50);

   // Display ON, cursor OFF, blink OFF
   lcd_control(0x0C);
   usDelay(50);

   // Increment mode, Entire shift off
   lcd_control(0x06);
   usDelay(50);

   // Clear display
   lcd_clear();
#endif
}

void lcd_home()
{
#if USE_LCD
   lcd_control(0x02);
   usDelay(1600);
#endif
}

void lcd_clear()
{
#if USE_LCD
   lcd_control(0x01);
   usDelay(1600);
#endif
}

void lcd_control(uchar controlByte)
{
#if USE_LCD
   // check if it's a clear screen command
   if((controlByte==0x01) || (controlByte&0xFE==0x02))
      lcdCharCount = 0;

   READ_WRITE = 0;
   DATA_CONTROL = 0;
   DATA_BUS = controlByte;

   // generate enable strobe
   ENABLE = 1;
   usDelay(2);
   ENABLE = 0;
#endif
}

void lcd_putchar(uchar dataByte)
{
#if USE_LCD
   READ_WRITE = 0;
   DATA_CONTROL = 1;
   DATA_BUS = dataByte;

   // generate enable strobe
   ENABLE = 1;
   usDelay(2);
   ENABLE = 0;

   usDelay(50);

   lcdCharCount++;
   if(lcdCharCount==8)
   {
      // set DDRAM address to next line
      lcd_control(40 + 0x80);
      usDelay(50);
   }
   else if(lcdCharCount==16)
   {
      // set DDRAM address to first line
      lcd_control(0x80);
      usDelay(50);
      lcdCharCount = 0;
   }
#endif
}


/*
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Function:	LCD_WriteMessage
;
; Desc:		Send a message string to the LCD. Homes the cursor to the left
; 			first and handles the mid-line barrier.
;
;			At this time, the LCD_WriteMessage function always begins with
;			the cursor at the home (upper-left-most) position. To write on
;			line #2, for example, you would start the message with a
;			line feed (LF) command byte. The LCD is homed on each new
;			message. The Form Feed (FF) command will clear the display.
;
*/
void lcd_putstring(uchar* message, uchar len)
{
#if USE_LCD
   do
   {
      lcd_putchar(*message);
      message++;
   }
   while(--len);
#endif
}
