//load420.c

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <termio.h>

#define DEFAULT_TIMEOUT 200
#define CRLF "\r\n"
#define buf_size 1024
#define BANNER "\r\nDS89C420 LOADER VERSION 1.0  COPYRIGHT (C) 2000 DALLAS SEMICONDUCTOR\r\n> \r\n\r\n> "
#define KLEAR_CMD "K\r\n"
#define KLEAR_RESP "K\r\n\r\n> "
#define LOAD_CMD "LB\r\n"
#define LOAD_RESP "LB\r\n"

#define DIE(s) {puts(s); exit(1);}


int comPortHandle = -1, fileHandle = -1;

void printHex(char* buf, int len)
{
   int i = 0;
   for(; i<len; i++)
   {
      printf("%02X ", buf[i]);
      if((((i+1)%16)==0))
         printf("\n");
   }
   if(((i+1)%16)!=0)
      printf("\n");
}

int writeCOM(char* buf, int len)
{
   int cnt = 0, tmpCnt;
   int timeout = DEFAULT_TIMEOUT;
   do
   {
      tmpCnt = write(comPortHandle, &buf[cnt], len-cnt);
      if(tmpCnt>0)
      {
         cnt += tmpCnt;
         timeout = DEFAULT_TIMEOUT;
      }
      else
      {
         --timeout;
         usleep(100);
      }
   }
   while(cnt<len && timeout);
   return cnt;
}

int readCOM(char* buf, int len)
{
   int cnt = 0, tmpCnt;
   int timeout = DEFAULT_TIMEOUT;
   do
   {
      tmpCnt = read(comPortHandle, &buf[cnt], len-cnt);
      if(tmpCnt>0)
      {
         cnt += tmpCnt;
         timeout = DEFAULT_TIMEOUT;
      }
      else
      {
         --timeout;
         usleep(100);
      }
   }
   while(cnt<len && timeout);
   return cnt;
}

char readCOMwait()
{
   char buf[1];
   while(read(comPortHandle, buf, 1)<=0)
      usleep(100);
   return buf[0];
}

int getBaudValue(int baud)
{
   switch(baud)
   {
      case 115200:
         return B115200;
      case 57600:
         return B57600;
      case 38400:
         return B38400;
      case 19200:
         return B19200;
      case 9600:
         return B9600;
      case 2400:
         return B2400;
   }
   DIE("Baud rate not supported");
}

int main(int argc, char** argv)
{
   char buf[buf_size];
   int cnt, i;
   int waitFlag = 1;
   struct termio oldterm, term;
   int baud = B57600;

   if (argc < 3)
   {
      DIE("\nUsage: load420 COM1 ecash.hex [57600] \n"
          "\n  Port and file required, baud is optional");
   }

   if(argc==4)
   {
      baud = getBaudValue(atoi(argv[3]));
   }

   // open the com port...
   comPortHandle  = open(argv[1], O_RDWR | O_NOCTTY | O_NONBLOCK | O_NDELAY);

   // Get the current options for the port...
   tcgetattr(comPortHandle, &term);
   // Make a backup of the current options, to restore later...
   tcgetattr(comPortHandle, &oldterm);

   //Set the baud rates...
   cfsetispeed(&term, baud);
   cfsetospeed(&term, baud);

   //Enable the receiver and set local mode...
   term.c_cflag |= (CLOCAL | CREAD);

   // Set the new options for the port...
   tcsetattr(comPortHandle, TCSAFLUSH, &term);

   // open the hex file...
   fileHandle = open(argv[2], O_RDWR | O_NOCTTY);

   puts("Getting loader prompt...");
   writeCOM(CRLF CRLF, 2*(sizeof(CRLF)-1));
   cnt = readCOM(buf, sizeof(BANNER)-1);
   if(memcmp(BANNER, buf, sizeof(BANNER)-1)!=0)
   {
      DIE("No 420 Loader found");
   }

   puts("Klearing memory...");
   writeCOM(KLEAR_CMD, sizeof(KLEAR_CMD)-1);
   cnt = 0;
   waitFlag = 1;
   while(waitFlag)
   {
      cnt += readCOM(&buf[cnt], sizeof(KLEAR_RESP)-1-cnt);
      if(cnt>=(sizeof(KLEAR_RESP)-1))
      {
         if(memcmp(KLEAR_RESP, buf, sizeof(KLEAR_RESP)-1)==0)
            waitFlag = 0;
      }
   }

   puts("Loading file...");
   writeCOM(LOAD_CMD, sizeof(LOAD_CMD)-1);
   cnt = 0;
   waitFlag = 1;
   while(waitFlag)
   {
      cnt += readCOM(&buf[cnt], sizeof(LOAD_RESP)-1-cnt);
      if(cnt>=(sizeof(LOAD_RESP)-1))
      {
         if(memcmp(LOAD_RESP, buf, sizeof(LOAD_RESP)-1)==0)
            waitFlag = 0;
      }
   }

   cnt = 0;
   waitFlag = 1;
   while(waitFlag)
   {

      int tmpCnt = read(fileHandle, &buf[cnt], buf_size-cnt);
      if(tmpCnt>0)
         cnt += tmpCnt;
      else if(cnt==0)
         waitFlag = 0;

      while(cnt>0)
      {
         for(i=0; i<cnt; i++)
         {
            if(buf[i]=='\n')
               break;
         }
         if(i<cnt)
         {
            char c = 0;
            i++;
            cnt -= i;

            write(comPortHandle, buf, i);

            c = readCOMwait();
            if(c!='G')
            {
               putchar(c);
               DIE("\nLoad failed\n");
            }
            if(cnt>0)
            {
               for(tmpCnt=0; tmpCnt<cnt; tmpCnt++)
                  if((tmpCnt+i)<buf_size)
                     buf[tmpCnt] = buf[tmpCnt+i];
                  else
                     buf[tmpCnt] = 0x00;
            }
         }
         else
         {
            break;
         }
      }
   }
   puts("Done loading");

   // close file
   close(fileHandle);

   // restore the old options for the port...
   tcsetattr(comPortHandle, TCSAFLUSH, &oldterm);

   // close com port
   close(comPortHandle);
}